#-------------------------------------------------------------#
#  Label Switching in Bayesian Mixture Models: deterministic  #
#                  relabeling strategies.                     #
#          by Carlos E. Rodriguez & Stephen G. Walker         #
#-------------------------------------------------------------#
#                                                             #
#   Directory where the folder LabelSwitching is              #
#  e.g. if the path is (this was our case)                    #
#                                                             #
#   /home/carloserwin/Documents/LabelSwitching/               #
#                                                             #
#  then                                                       #
#                                                             #
#  dir_work <- '/home/carloserwin/Documents/LabelSwitching/'  #
#                                                             #
dir_work <- '/home/carloserwin/Desktop/LabelSwitching/'
#                                                             #
#           LOAD THE FUNCTIONS TO USE                         #
#   SEE THE FILE Functions.r FOR A COMPLETE DESCRIPTION.      #
#  ALL THE FUNCTIONS HAVE A HEADER AND A NUMBER, THIS MAKES   #
#  IT EASY TO GO FROM THIS FILE TO THE FILE Functions.r AND   #
#  READ HOW TO USE EACH FUNCTION. INSTRUCTIONS TO ADD YOUR    #
#  OWN DATA ARE INCLUDED IN THE SAME FILE: Functions.r        # 
#                                                             #
source(paste(dir_work, 'Functions.r', sep =''))
#                                                             #
#           LOAD THE DYNAMIC LIBRARIES                        #
# COMPILE AND GENERATE THE SHARED LIBRARIES BEFORE THIS STEP  # 
#                                                             #
dyn.load(paste(dir_work, 'gibbs/univ_gibbs.so', sep=''))
dyn.load(paste(dir_work, 'relabel/relabel.so', sep=''))
#                                                             #
#   Directory where the data sets to analyze should be        #
#                                                             #
dir_data  <- paste(dir_work, 'data_sets/', sep ='')
#------------------------------------------------------------ #
#                                                             #
#   Directory where the results will be stored                #
#                                                             #
dir  <- paste(dir_work, 'results/', sep ='')
#-------------------------------------------------------------#
#           1- Function Loading Data                          #
#-------------------------------------------------------------#
#                                                             #
#    This function loads a data set with the structure        #
#           needed to implement our programs                  #
#                                                             #
# type    - 1-> data from model (4.1) pp 322 from a)          # 
#           2-> data from model (4.2) pp 322 from a)          #
#               and Model 3 from d)                           #
#           3-> the famous galaxy data from c) and others     #
#           4-> the acidity data from c)                      # 
#           5-> the enzyme data from c)                       #
#           6-> the famous crab data generated                #
#               following b)                                  #
#           7-> data from Model 1, a version of the           #  
#               crab data from d)                             #
#           8-> data from Model 2 from d)                     #
#           9-> add your data                                 #
#                                                             #
# a) Papastamoulis, P., and Iliopoulos, G. (2010)             #
# b) Pearson, K.(1894)                                        #
# c) Richardson, S., and Green, P. J.(1997)                   #
# d) Rodriguez, C. E. and Walker, S. G. (2012)                # 
#                                                             #                   
type <- 3
n_star <- 500    
in_out <- input_output_load(type, n_star, dir_data, dir)
#                                                             #
#            Independent constants                            #
#                                                             #
#set.seed(3363322)  # uncomment to fix a random seed
m <- 60000          # Total number of iterations
burn <- 30000       # Burn in period, NOTE THAT: m > burn 
m2 <- m - burn      # Iterations that will be stored (after the burn in period)
#-------------------------------------------------------------#
#           2- Storage Requirements                           #
#-------------------------------------------------------------#
TFS <- storage_requirements(m2, in_out$k, in_out$y)
TFS
#-------------------------------------------------------------#
#                 Data based priors of                        # 
#         Richardson, S., and Green, P. J.(1997)              #
#-------------------------------------------------------------#
delta <- 1.0
R <- max(in_out$y) - min(in_out$y)
R2 <- R*R
mu0 <- min(in_out$y) + R/2
kappa <- 1.0/R2
alpha <- 2.0
gg <- 0.2
hh <- (100.0*gg)/(alpha*R2)
#-------------------------------------------------------------#
#              3- Function Standard Gibbs                     #
#-------------------------------------------------------------#
tt0 <- system.time(normal_gibbs(burn, m2, in_out$k, in_out$y, delta, mu0, kappa, alpha, gg, hh, 
                                in_out$dir_res))[3]
tt0/60 # system time in minutes
#-------------------------------------------------------------#
#   4-  Function Representatives for the Data Algorithm       #
#-------------------------------------------------------------#   
tt_gr <- system.time(mygr <- representatives_mean_var(in_out$k, m2, in_out$y, in_out$dir_res))[3]
tt_gr # system time in seconds
#-------------------------------------------------------------#
#         5-  Function MAP estimate for d                     #
#-------------------------------------------------------------#
tt_map <- system.time(z_map <- find_z_MAP(in_out$k, in_out$y, in_out$dir_res))[3]
tt_map # system time in seconds
#-------------------------------------------------------------#
#  6-  Interface to the relabeling algorithms: Data strategy  #
#-------------------------------------------------------------#
tt_Data <- relabel(option = 'Data', in_out$k, m2, in_out$y, mu = mygr$mu, sigma2 = mygr$sig2,  
                   dir = in_out$dir_res, label_name = 'label_Data.txt')
tt_Data # system time in seconds
#-------------------------------------------------------------#
#  6-  Interface to the relabeling algorithms: ECR strategy   #
#-------------------------------------------------------------#
tt_ECR <- relabel(option ='ECR', in_out$k, m2, in_out$y, z_map = z_map, dir = in_out$dir_res, 
                  label_name = 'label_ECR.txt')
tt_ECR # system time in seconds
#-------------------------------------------------------------#
#  6-  Interface to the relabeling algorithms:  KL strategy   #
#-------------------------------------------------------------#
tt_KL <- relabel(option = 'KL', in_out$k, m2, in_out$y, rnd = 0, epsilon = 100, cont = 30, 
								 prob_name = 'pclass_KL.txt', dir = in_out$dir_res, label_name = 'label_KL.txt')
tt_KL/60 # system time in minutes 
#------------------------------------------------------------#
#           7-  Posterior Mean Estimates                     #
#------------------------------------------------------------#
post_mean_Data <- posterior_mean_estimates(in_out$k, m2, in_out$dir_res, 'label_Data.txt')
post_mean_Data$w
post_mean_Data$mu
post_mean_Data$sig2
#------------------------------------------------------------#
#             8 - Reference Parameters                       #
#------------------------------------------------------------#
ref <- ref_values(in_out$type)
#ref <- post_mean_Data # if there are no reference parameters
#------------------------------------------------------------#
#       9 - Comparison of Posterior Mean Estimates           #
#------------------------------------------------------------#
COMPAR  <- compar_posterior_mean_estimates(in_out$k, m2, in_out$dir_res, ref, 
                                           'label_Data.txt', 'label_ECR.txt', 'label_KL.txt')
round(COMPAR$W, 4)
round(COMPAR$MU, 3)
round(COMPAR$SIGMA2, 6)
#------------------------------------------------------------#
#          10 -  Scaled Components Predictive                #
#------------------------------------------------------------#
tt1 <- system.time(scaled_comp_pred(in_out$k, m2, in_out$y_star, in_out$dir_res, 
                                               'label_Data.txt', 'pred_Data.txt'))[3]
tt2 <- system.time(scaled_comp_pred(in_out$k, m2, in_out$y_star, in_out$dir_res, 
                                               'label_ECR.txt', 'pred_ECR.txt'))[3]
tt3 <- system.time(scaled_comp_pred(in_out$k, m2, in_out$y_star, in_out$dir_res, 
                                               'label_KL.txt', 'pred_KL.txt'))[3]
#------------------------------------------------------------#
#            11-  Plotting the Predictive                    #
#------------------------------------------------------------#
my_pred_graph(in_out, 'pred_Data.txt')
#------------------------------------------------------------#
#     12 -  Posterior Classification Probabilities           #
#------------------------------------------------------------#
tt1b <- system.time(class(in_out$k, m2, in_out$y, in_out$dir_res, 'label_Data.txt', 'pclass_Data.txt'))
tt2b <- system.time(class(in_out$k, m2, in_out$y, in_out$dir_res, 'label_ECR.txt', 'pclass_ECR.txt'))
#------------------------------------------------------------#
#    13 - Plotting the Scaled Components Predictive          #
#               and Single Best Clustering                   #
#------------------------------------------------------------#
my_scaled_clust_graph(in_out$k, in_out, 'pred_Data.txt', 'pclass_Data.txt', 0)
#------------------------------------------------------------#
#    13 - Plotting the Scaled Components Predictive          #
#               and Single Best Clustering                   #
#              COMPARISON BETWEEN ALGORITHMS                 #
#------------------------------------------------------------#
par(mfrow = c(1, 3))
my_scaled_clust_graph(in_out$k, in_out, 'pred_Data.txt', 'pclass_Data.txt', 'Data')
my_scaled_clust_graph(in_out$k, in_out, 'pred_ECR.txt', 'pclass_ECR.txt', 'ECR')
my_scaled_clust_graph(in_out$k, in_out, 'pred_KL.txt', 'pclass_KL.txt', 'KL')	
#------------------------------------------------------------#
#               14 - Plotting the Trace                      #
#------------------------------------------------------------#
mytrace(option = 'means', in_out$k, m2, in_out$dir_res, 'label_Data.txt', 0)
mytrace(option = 'weights', in_out$k, m2, in_out$dir_res, 'label_Data.txt', 0)
mytrace(option = 'variances', in_out$k, m2, in_out$dir_res, 'label_Data.txt', 0)
#------------------------------------------------------------#
#               14 - Plotting the Trace                      #
#              COMPARISON BETWEEN ALGORITHMS                 #
#------------------------------------------------------------#
par(mfrow = c(1, 3))
mytrace(option = 'means', in_out$k, m2, in_out$dir_res, 'label_Data.txt', 'Data')
mytrace(option = 'means', in_out$k, m2, in_out$dir_res, 'label_ECR.txt', 'ECR')
mytrace(option = 'means', in_out$k, m2, in_out$dir_res, 'label_KL.txt', 'KL')
#                                                            #
#             Remove everything from R                       # 
#   Note that the text files will remain in their respective #
#   folder and to display the results you will not need to   #
#   run the all programs again!                              #
#                                                            #
rm(list = ls())
