#include "utilities.h"
/* Generate a random variable Y~Gamma(alpha, beta) */
/* parametrized such that:                         */
/*              E[Y] = alpha/beta                  */
double rmygamma(double alpha, double beta){
  return rgamma(alpha, 1.0/beta);
}
/*   Extract randomly one element of {0, 2, ..., k-1} with   */
/*   probabilities p the probabilities in p does not sum up  */
/*   to one but divided by scale_p they do sum 1 (burn)      */
int sample_probs_burn(int k, int *perm, double *p, double scale_p){

  double u;
  int j, km1 = k - 1;
  /* Sort probabilities into descending order */
  /* Order element identities in parallel */
  revsort(p, perm, k);  
  /* compute cumulative probabilities */
  p[0] /= scale_p;
  for(j = 1 ; j < k; j++){
    p[j] /= scale_p;
    p[j] += p[j-1];
  }  
  /* compute the sample */
  u = unif_rand();
  for(j = 0; j < km1; j++){
    if(u <= p[j]){
      break;
    }
  }
  return perm[j];
}
/*   Extract randomly one element of {0, 2, ..., k-1}  */
/*                  p does sum 1                       */
int sample_probs(int k, int *perm, double *p){

  double u;
  int j, km1 = k - 1;
  /* Sort probabilities into descending order */
  /* Order element identities in parallel */
  revsort(p, perm, k);
  /* compute cumulative probabilities */
  for(j = 1 ; j < k; j++){
    p[j] += p[j-1];
  }
  /* compute the sample */
  u = unif_rand();
  for(j = 0; j < km1; j++){
    if(u <= p[j]){
      break;
    }
  }
  return perm[j];
}
/* ------------------------------------------------- */
/* Updating allocations for the Gibbs sampler (burn) */
/* ------------------------------------------------- */
void sample_allocations_burn(int k, int n, int *z, double *y, double *w, double *mu,
			     double *tau, double *sqrt_tau, int *nj, double *y_sum,
			     double *y2_sum){
  int i, j, perm[k];
  double pz[k], scale_pz;

  for(i = 0; i < n; i++){
    scale_pz = 0.0;
    for(j = 0; j < k; j++){      
			pz[j] = w[j]*sqrt_tau[j]*exp(-0.5*tau[j]*(y[i] - mu[j])*(y[i] - mu[j]));
      scale_pz += pz[j];
      perm[j] = j;
    }
    z[i] = sample_probs_burn(k, perm, pz, scale_pz);
    nj[z[i]]++;
    y_sum[z[i]] += y[i];
    y2_sum[z[i]] += y[i]*y[i];
  }
}
/* ------------------------------------------ */
/* Updating allocations for the Gibbs sampler */
/* ------------------------------------------ */
void sample_allocations(int k, int n, int *z, double *y, double *w, double *mu,
			double *tau, double *sqrt_tau, int *nj, double *y_sum,
			double *y2_sum, FILE *ffile_z, FILE *ffile_pz){
  int i, j, perm[k];
  double pz[k], scale_pz;

  for(i = 0; i < n; i++){
    scale_pz = 0.0;
    for(j = 0; j < k; j++){
			pz[j] = w[j]*sqrt_tau[j]*exp(-0.5*tau[j]*(y[i] - mu[j])*(y[i] - mu[j]));
      scale_pz += pz[j];
      perm[j] = j;      
    }
    for(j = 0; j < k; j++){
      pz[j] /= scale_pz;
			fprintf(ffile_pz, "%12.16f \t", pz[j]);
    }
    fprintf(ffile_pz, "\n");    
    z[i] = sample_probs(k, perm, pz);
    fprintf(ffile_z, "%d \t", z[i]);
    nj[z[i]]++;
    y_sum[z[i]] += y[i];
    y2_sum[z[i]] += y[i]*y[i];
  }
  fprintf(ffile_z, "\n");
}